clear;format bank

%  Factor & Extension Analysis Program  (MATLAB version).

%  primary sources: 
%  Gorsuch, R. L. (1997a). Exploratory factor analysis: Its role 
%  in item analysis. Journal of Personality Assessment, 68, 532-560
   
%  Gorsuch, R. L. (1997b). New procedure for extension analysis in 
%  exploratory factor analysis. Educational and Psychological 
%  Measurement, 57, 725-740.


% To specifiy your options, repond to the queries at the 
% start of the program.


%  Prepare a raw data matrix for analysis,
%  where rows = cases, & columns = variables;
%  Cases with missing values are not permitted in the data file;
%  The core variables must be the first x variables in the data file;
%  The extension variables must appear after (to the left of) 
%  the core variables in the data file.

%  Use the following name for the raw data matrix: "data"

%  For the trial analyses, the program was set to read the raw data 
%  file named "trial.dat", and then process this data file. 
load trial.dat;  data = trial;


% How many core variables are there? 
% The core variables must be the first x variables in the data file
core = 14;

% How many extension variables are there?
% The extension variables must appear after (to the right of) the core variables in the data file
ext  = 4;

% What kind of extension analysis do you want to run?
% Enter "1" for regular extension analysis
% Enter "2" for higher order factor/extension analysis
kind = 2;

% Specify a procedure for determining the number of factors:
% Enter "1" for number of salient loadings (Gorsuch, 1997, J Pers Assess, p 545)
% Enter "2" for parallel analysis of principal components (Zwick & Velicer, 1986, Psych Bull, 99, p 432)
% Enter "3" for parallel analysis of principal axis / common factors
% Enter "4" for for Velicer's MAP test (Velicer, 1976, Psychometrika, 41, 321-327)
% Enter "5" for Standard Error Scree test (Zoski & Jurs, 1996, EPM, p 443)
% Enter "6" for number of eigenvalues > 1
% Enter "7" for a user-selected number of factors (& see the next 2 commands)
roottest = 6;

% If "7" was specified for roottest, then enter the desired number of factors
nfactors = 4;

% If "7" was specified for roottest, then enter the desired number of factors
% for the higher order analysis (if requested)
nfactsho = 2;

% Specify a factor extraction procedure:
% Enter "1" for common factor analysis
% Enter "2" for principal components
% Enter "3" for maximum likelihood factor analysis
% Enter "4" for image analysis
extract = 2;

% Specify a rotation procedure:
% Enter "1" for promax
% Enter "2" for varimax     (not appropriate for higher order analyses)
% Enter "3" for no rotation (not appropriate for higher order analyses)
rotate = 1;


% End of required user specifications


% The following additional specifications can also be made (or ignored)

% Parallel analysis: Enter the # of random data sets
Ndatsets = 100;

% Parallel analysis: Enter the desired percentile eigenvalue (default = 95th)
percent = 95;

% Number of Salient Loadings: 
% Enter the minimum abs value for a "salient" loading (default = .40)
salvalue = .40;

% Number of Salient Loadings: 
% Enter the minimum # of salient loadings for there to be "a factor" (default = 3)
numsals = 3;

% Common Factor analysis: maximum number of iterations (default = 30)
itercfa = 3;

% Maximum Likelihood Factor analysis: maximum number of iterations (default = 30)
iterml = 30;

% Maximum Likelihood Factor analysis: tolerance for loadings (default = .001)
tolerml = .001;

% Promax rotation: the power or k value (default = 3)
ppower = 3;

% Promax rotation: 
% Enter "1" for the structure matrix (default, & recommended by Gorsuch, 1997, p 556)
% Enter "2" for the pattern matrix
loadmat = 1;

% End of optional specifications



data(any(isnan(data)'),:) = [];
cases = size(data,1); 
nvars = size(data,2);

disp([' ']);disp([' ']);disp(['Extension Factor Analysis']);disp([' ']);
disp(['Number of Cases in the Raw Data Matrix = ' num2str(cases) ]);disp([' ']);
disp(['Total Number of Variables in the Raw Data Matrix = ' num2str(nvars) ]);
disp([' ']);disp(['Number of Core Variables = ' num2str(core) ]);disp([' ']);
disp(['Number of Extension Variables = ' num2str(ext) ]);disp([' ']);

% Improper specification warnings
if (core+ext) > nvars;disp(['  ']);
disp(['WARNING: More core and/or extension variables were specified than'])
disp(['there are variables in the raw data matrix: Expect error messages']);
disp(['  ']);end
if ( nvars > (core+ext) );disp(['  ']);
disp(['WARNING: There are more variables in the raw data']) 
disp(['than there are core & extension variables.  The data']) 
disp(['matrix was trimmed to core+extension variables.']);disp(['  ']);
data = data(:,1:(core+ext));
nvars = size(data,2);
end

if roottest == 1; ntype = '# Salient Loadings';end
if roottest == 2; ntype = 'Parallel Analysis of Principal Components';end
if roottest == 3; ntype = 'Parallel Analysis of Principal Axis / Common Factors';end
if roottest == 4; ntype = 'Velicers MAP test';end
if roottest == 5; ntype = 'Standard Error Scree';end
if roottest == 6; ntype = '# Eigenvalues > 1';end
if roottest == 7; ntype = 'User-Specified';end
disp(['Test for Number of Factors:   ' ntype ]);disp([' ']);

if extract == 1; etype = 'Common Factor Analysis';end
if extract == 2; etype = 'Principal Components';end
if extract == 3; etype = 'Maximum Likelihood';end
if extract == 4; etype = 'Image Analysis';end
disp(['Factor Extraction Procedure:  ' etype ]);disp([' ']);

if rotate == 1; rtype = 'Promax';end
if rotate == 2; rtype = 'Varimax';end
if rotate == 3; rtype = 'No Rotation';end
disp(['Rotation Procedure:           ' rtype ]);disp([' ']);

 
rdata = corrcoef([ data ]);

rcore =  rdata(1:core,1:core) ;  r = rcore; evals1 = (sort(eig(r)*-1))*-1; 


% Number of Factors
if roottest == 1; salient;        nfactors = nfSAL;end
if roottest == 2; parallelPCA;    nfactors = nfPARPCA;end
if roottest == 3; parallelPAF;    nfactors = nfPARPAF;end
if roottest == 4; velmap ;        nfactors = nfVEL;end
if roottest == 5; sescree;        nfactors = nfSCREE;end
if roottest == 6; nevalsgt1;      nfactors = nfnevalsgt1;end
warnnf1 = 0;
if nfactors == 0;                 nfactors = 1; warnnf1 = 1;end

% Factor Extraction
if extract == 1; cfa;     end
if extract == 2; pca;     end
if extract == 3; maxlike; end
if extract == 4; imagefa; end

% Rotation, which also gives the factor intercorrelations
if nfactors == 1;         rff = 1   ;end
if nfactors  > 1;
if rotate == 1; promax;   rff = Phip;end
if rotate == 2; varimax;            ;end;end

nfactors1 = nfactors;


% for regular extension analysis
% supermatrix: core variables, extension variables, & the factor loadings
sup = repmat(0, (size(rdata,2)+nfactors) , (size(rdata,2)+nfactors) );
sup(1:size(rdata,1),1:size(rdata,2)) = rdata;
sup( (size(sup,1)-(nfactors-1)):size(sup,1), 1:size(lding,1) ) = lding(1:core,:)';


% correlations between extension variables & the factors
ppp = repmat(0, size(sup,1) , size(sup,2) );
resid = sup;
for lupe  = 1:core
for luper = lupe:size(sup,1)
ppp(luper,lupe) = resid(luper,lupe) / sqrt(resid(lupe,lupe));
end;
resid =  resid - ppp(:,lupe) * ppp(:,lupe)';end
pvo = ppp(1:core,1:core);
peo = ppp(core+1:(core+ext),1:core);
pfo = ppp( (core+ext+1):(size(ppp,1)) , 1:core);
sef1 = peo * pfo';



% for higher order factor/extension analysis
if kind == 2 & nfactors1 > 1 & rotate == 1;
lding1 = lding; core1 = core; ext1 = ext; 
r = rff;  evals2 = (sort(eig(r)*-1))*-1;

% Number of Factors
if roottest == 1; salient;        nfactors = nfSAL;end
if roottest == 2; parallelPCA;    nfactors = nfPARPCA;end
if roottest == 3; parallelPAF;    nfactors = nfPARPAF;end
if roottest == 4; velmap ;        nfactors = nfVEL;end
if roottest == 5; sescree;        nfactors = nfSCREE;end
if roottest == 6; nevalsgt1;      nfactors = nfnevalsgt1;end
if roottest == 7;                 nfactors = nfactsho;end
warnnf2 = 0;
if nfactors == 0;                 nfactors = 1; warnnf2 = 1;end

% Factor Extraction
if extract == 1; cfa;     end
if extract == 2; pca;     end
if extract == 3; maxlike; end
if extract == 4; imagefa; end

% Rotation
if nfactors  > 1;if rotate == 1; promax;  end;end


% supermatrix: core variables, extension variables, & the factor loadings
sup = repmat(0, (nfactors1+core+ext+nfactors) , (nfactors1+core+ext+nfactors) );

% the core variables for the higher order analysis = the factor intercorrelations
sup(1:size(rff,1),1:size(rff,2)) = rff;

% the first set of extension variables
sup( (nfactors1+1):(nfactors1+core), 1:(size(lding1,2)) ) = lding1;

% the possible additional extension variables
if ext > 0;
sup( (nfactors1+core+1):(nfactors1+core+ext) , 1:nfactors ) =  ...
     rdata( (core+1):(core+ext), 1:nfactors );end

% the loadings (correlations) from the higher order factor analysis
sup( (size(sup,1)-(nfactors-1)):size(sup,1), 1:size(lding,1) ) = lding';

ext = core + ext;  core = nfactors1; 



% correlations between extension variables & the higher order factors
ppp = repmat(0, size(sup,1) , size(sup,2));

resid = sup;
for lupe  = 1:core
for luper = lupe:size(sup,1)
ppp(luper,lupe) = resid(luper,lupe) / sqrt(resid(lupe,lupe));
end;
resid =  resid - ppp(:,lupe) * ppp(:,lupe)';end
pvo = ppp(1:core,1:core);
peo = ppp(core+1:(core+ext),1:core);
pfo = ppp( (core+ext+1):(size(ppp,1)) , 1:core);
sef = peo * pfo';

end


if kind == 1 | ( kind == 2 & nfactors1 == 1) | ( kind == 2 & rotate > 1)
disp(['Eigenvalues for the Core Variables:']);disp([' ']);
disp([(1:size(evals1,1))' evals1]);disp([' ']);
if warnnf1 == 1;disp(['  ']);
disp(['WARNING: Zero factors were found in the data.']);
disp(['         The number of factors was therefore set at 1.']);disp(['  ']);end
disp(['Number of Factors = ' num2str(nfactors) ]);disp([' ']);
if nfactors1 == 1;
disp(['WARNING: There was only one factor, rotation not performed']);disp(['  ']);end
if rotate == 1 & nfactors1 > 1;disp(['Factor Intercorrelations:']);disp(rff);end
disp(['Core Variable Loadings on the Factors:']);disp([' ']);
disp([(1:size(lding,1))' lding])
if ext > 0
disp(['Extension Variable Correlations with the Factors']);disp([' ']);end
disp([ (1:size(sef1,1))' sef1 ])
end


% Warning message
if kind == 2 & nfactors1 == 1;
disp(['WARNING: Higher Order Factor/Extension Analysis was requested, but there was'])
disp(['         only one factor in the lower-order data.'])  
disp(['         The higher order analyses were not performed.']);disp(['  ']);end

% Warning message
if kind == 2 & rotate > 1;
disp(['WARNING: Higher Order Factor/Extension Analysis was requested, as was'])
disp(['         a ROTATE option that does not produce correlations between between factors.'])
disp(['         The higher order analyses were not performed.']);disp(['  ']);end


if kind == 2 & nfactors1 > 1 & rotate == 1
disp(['Eigenvalues for the First Set of Core Variables:']);
disp([' ']);disp([(1:size(evals1,1))' evals1]);disp([' ']);
disp(['Number of Factors in the First Set of Core Variables = ' num2str(nfactors1) ]);
disp([' ']);disp(['Eigenvalues for the Higher Order Factor Analysis:']);
disp([' ']);disp([(1:size(evals2,1))' evals2]);disp([' ']);
if warnnf2 == 1;disp(['  ']);
disp(['WARNING: Zero factors were found in the higher order data.']);
disp(['         The number of factors was therefore set at 1.']);disp(['  ']);end
disp(['Number of Factors for the Higher Order Factor Analysis = ' num2str(nfactors) ]);
disp([' ']);
if nfactors == 1;
disp(['WARNING: There was only one higher order factor, rotation not performed']);
disp(['  ']);end
disp(['Factor Intercorrelations from the First Factor Analysis and, in the'])
disp(['far-right collumn(s), the Item Loadings on the Higher Order Factor(s):']);disp([' ']);
disp([ (1:size(lding,1))' rff  lding]);disp([' ']);
disp(['Core Variable Loadings on the Lower Order Factors and, in the '])
disp(['far-right collumn(s), their Correlations with the Higher Order Factor(s):']);disp([' ']);
disp([ (1:size(lding1,1))'  lding1   sef(1:core1,:)]);disp([' ']);
if ext1 > 0
disp(['Extension Variable Correlations with the Lower Order Factor(s) and, in the']);
disp(['far-right collumn(s), their Correlations with the Higher Order Factor(s):']);disp([' ']);
disp([' ']);disp( [ (1:ext1)' sef1 sef(core1+1:size(sef,1),:) ]);end
end







